import arcpy
import sys
import csv
import requests
import time
import shutil

#get input database from arcmap user input
tablePath = arcpy.GetParameterAsText(0)
address = arcpy.GetParameterAsText(1)
city = arcpy.GetParameterAsText(2)
state = arcpy.GetParameterAsText(3)
zip = arcpy.GetParameterAsText(4)
apikey = arcpy.GetParameterAsText(5)
naaccrMember = arcpy.GetParameter(8)
addressIndex = 0
cityIndex = 0
stateIndex = 0
zipIndex = 0

    
arcpy.AddMessage("version: " + sys.version)

if(arcpy.GetParameter(7)):
    fields_to_add = ["Latitude","Longitude","MatchType","Result","Block","Group","Tract","Cnty_Fips","CBSA_Fips","CBSA_Micro","MCD_Fips","MetDiv","MSA_Fips","Place_Fips","State_Fips","Group"]
else:
    fields_to_add = ["Latitude","Longitude","MatchType","Result"]    

fieldList = arcpy.ListFields(tablePath)
arcpy.AddWarning("fieldList length: "+str(len(fieldList)))
fieldName = [f.name for f in fieldList]


#region csv

if (tablePath.endswith('.csv')) or (tablePath.endswith('.txt')):
    count = 0
    for field in fieldName:
        if field == address:
            addressIndex = count
        if field == city:
            cityIndex = count
        if field == state:
            stateIndex = count
        if field == zip:
            zipIndex = count
        count = count +1
    arcpy.AddWarning("Processing csv file...")
    csvPathOut = tablePath[:-4]
    if tablePath.endswith('.txt'):
        newTable = csvPathOut +'_Results.txt'
    else:
        newTable = csvPathOut +'_Results.csv'
    with open(tablePath,'r') as csvinput:
        with open(newTable,'w') as csvoutput:
            totalRows = int(arcpy.GetCount_management(tablePath).getOutput(0))
            arcpy.AddWarning("Processing "+str(totalRows)+" total records...")
            #this sets up the progress bar
            arcpy.SetProgressor("step", "Copying shapefiles to geodatabase...",0, totalRows, 1)
            start_time = time.time()
            success = 1
            writer = csv.writer(csvoutput, lineterminator='\n')

            all = []
            if tablePath.endswith('.txt'):
                reader = csv.reader(csvinput, delimiter='\t')
            else:
                reader = csv.reader(csvinput)
            row = next(reader)
            for field in fields_to_add:
                if field not in fieldName:
                    row.append(field)
            all.append(row)            
            for row in reader:
                arcpy.SetProgressorLabel("Loading {0}...".format(row))
                #myList = list(row)
                #arcpy.AddError(myList)
                if naaccrMember:
                    url = "https://geo.naaccr.org/Services/Geocode/WebService/GeocoderWebServiceHttpNonParsed_V04_01.aspx?"
                else:                    
                    url = "https://geoservices.tamu.edu/Services/Geocode/WebService/GeocoderWebServiceHttpNonParsed_V04_01.aspx?"

                payload = "streetAddress="+str(row[addressIndex])+"&city="+str(row[cityIndex])+"&state="+str(row[stateIndex])+"&zip="+str(row[zipIndex])+"&apikey="+apikey+"&format=csv&census=true&censusYear=2010&notStore=false&version=4.01&source=Esri"

                headers = {
                        'accept': "application/x-www-form-urlencoded",
                        'content-type': "application/x-www-form-urlencoded",
                        'access-control-allow-origin': "*"
                        }

                response = requests.request("POST", url, data=payload, headers=headers)               
                length = len(response.text)
                if length>0:
                    responseList = response.text.split(',')
                else:
                    arcpy.AddWarning("There are 0 results")
                    arcpy.AddError("This is likely due to an invalid api key")
                    success = 0
                    sys.exit(1)
                responseListOut = []                
                row.append(responseList[3])
                row.append(responseList[4])
                row.append(responseList[11])
                row.append(responseList[9])
                if(arcpy.GetParameter(7) and responseList[9] == "Success"):
                    row.append(responseList[18])
                row.append(responseList[19])
                row.append(responseList[20])
                row.append(responseList[21])
                row.append(responseList[22])
                row.append(responseList[23])
                row.append(responseList[24])
                row.append(responseList[25])
                row.append(responseList[26])
                row.append(responseList[27])
                row.append(responseList[28])
                all.append(row)
                arcpy.SetProgressorPosition()           

            writer.writerows(all)
            end_time = (time.time()-start_time)
            if(success == 1):
                arcpy.AddMessage(" ")
                arcpy.AddMessage("Processing complete: " + str(totalRows) + " records")
                arcpy.AddMessage("Processing speed: " + str("%.2f" % (totalRows/end_time)) + " records/per second")
                arcpy.AddMessage(" ")
            else:
                arcpy.AddWarning("Please ensure your api key is entered correctly")
                sys.exit(1)
        arcpy.ResetProgressor()
        if(arcpy.GetParameter(6)):
                    arcpy.AddMessage("User elected to create display layer")
                    try:
                        x_coords = "Longitude"
                        y_coords = "Latitude"
                        z_coords = "POINT_Z"
                        out_Layer = "Data_Points_new"
                        saved_Layer = r"Data_Points_new"

                        # Set the spatial reference
                        spRef = r"Coordinate Systems\Geographic Coordinate System\World\WGS 1984"

                        # Make the XY event layer...
                        for i in range(1,10):
                            try:
                                arcpy.MakeXYEventLayer_management(newTable, x_coords, y_coords, out_Layer, spRef)
                                arcpy.SaveToLayerFile_management(out_Layer, saved_Layer)
                                mxd = arcpy.mapping.MapDocument("CURRENT")
                                dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                addlayer = arcpy.mapping.Layer(out_Layer)
                                break
                            except:
                                if("ERROR 000725" in str(arcpy.GetMessage(3)) ):
                                    saved_Layer_new = saved_Layer+"_"+str(i)
                                    out_Layer_new = out_Layer+"_"+str(i)
                                    try:
                                        arcpy.MakeXYEventLayer_management(newTable, x_coords, y_coords, out_Layer_new, spRef)
                                        arcpy.SaveToLayerFile_management(out_Layer_new, saved_Layer_new)
                                        mxd = arcpy.mapping.MapDocument("CURRENT")
                                        dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                        addlayer = arcpy.mapping.Layer(out_Layer_new)
                                        break
                                    except:
                                        if(i==1):
                                            arcpy.AddMessage("Data Layer "+ out_Layer + " already exists, trying: "+out_Layer_new)
                                            arcpy.AddMessage(" ")
                                        else:
                                            arcpy.AddMessage("Data Layer "+ out_Layer_new + " already exists, trying: "+out_Layer+"_"+str(i+1))
                                            arcpy.AddMessage(" ")

                        # Save to a layer file
                        arcpy.mapping.AddLayer(dataFrame, addlayer, "TOP")
                    except:
                        #if error occured print message to screen
                        arcpy.AddWarning("Unable to add layer - unknown reason")
                        arcpy.AddMessage(arcpy.GetMessages())
                    else:
                        arcpy.AddMessage("User elected not to create display layer")
    sys.exit(1)
#endregion

#region dbf
elif tablePath.endswith('.dbf'):
    shutil.copy2(tablePath,tablePath[:-4]+"_Results.dbf")
    tablePathNew = tablePath[:-4]+"_Results.dbf"
    arcpy.AddWarning("Processing dbf file...")
    for field in fields_to_add:
        if field not in fieldName:
            if field == "Latitude" or field =="Longitude":
                arcpy.AddField_management(tablePathNew,field,"FLOAT",15,15,"","","NULLABLE","NON_REQUIRED","");
            else:
                arcpy.AddField_management(tablePathNew,field,"string","","",55,"","NULLABLE","NON_REQUIRED","");

    fields_to_pass = []


    if(arcpy.GetParameter(7)):
        fields_to_pass = [address,city,state,zip,'Latitude','Longitude','MatchType','Result',"Block","Group","Tract","Cnty_Fips","CBSA_Fips","CBSA_Micro","MCD_Fips","MetDiv","MSA_Fips","Place_Fips","State_Fips","Group"]

    else:
        fields_to_pass = [address,city,state,zip,'Latitude','Longitude','MatchType','Result']

    with arcpy.da.UpdateCursor(tablePathNew, fields_to_pass) as cursor:
        totalRows = int(arcpy.GetCount_management(tablePathNew).getOutput(0))
        arcpy.AddWarning("Processing "+str(totalRows)+" total records...")
        #this sets up the progress bar
        arcpy.SetProgressor("step", "Copying shapefiles to geodatabase...",0, totalRows, 1)
        start_time = time.time()
        success = 1
        for row in cursor:
                arcpy.SetProgressorLabel("Loading {0}...".format(row))
                url = "https://geoservices-api.tamu.edu/Services/Geocode/WebService/GeocoderWebServiceHttpNonParsed_V04_01.aspx?"

                payload = "streetAddress="+str(row[0])+"&city="+str(row[1])+"&state="+str(row[2])+"&zip="+str(row[3])+"&apikey="+apikey+"&format=csv&census=true&censusYear=2010&notStore=false&version=4.01&source=Esri"

                headers = {
                        'accept': "application/x-www-form-urlencoded",
                        'content-type': "application/x-www-form-urlencoded",
                        'access-control-allow-origin': "*"
                        }

                response = requests.request("POST", url, data=payload, headers=headers)
                length = len(response.text)
                if length>0:
                        responseList = response.text.split(',')
                else:
                        arcpy.AddWarning("There are 0 results")
                        arcpy.AddError("This is likely due to an invalid api key")	
                        success = 0
                        sys.exit(1)			
                responseListOut = []
                row[4] = responseList[3]
                row[5] = responseList[4]
                row[6] = responseList[11]
                row[7] = responseList[9]
                if(arcpy.GetParameter(7) and responseList[9] == "Success"):
                    row[8] = responseList[40]
                    row[9] = responseList[41]
                    row[10] = responseList[42]
                    row[11] = responseList[43]
                    row[12] = responseList[44]
                    row[13] = responseList[45]
                    row[14] = responseList[46]
                    row[15] = responseList[47]
                    row[16] = responseList[48]
                    row[17] = responseList[49]
                    row[18] = responseList[50]
                cursor.updateRow(row)
                # the following updates the progress bar
                arcpy.SetProgressorPosition()
        end_time = (time.time()-start_time)
        if(success == 1):
            arcpy.AddMessage(" ")
            arcpy.AddMessage("Processing complete: " + str(totalRows) + " records")
            arcpy.AddMessage("Processing speed: " + str("%.2f" % (totalRows/end_time)) + " records/per second")
            arcpy.AddMessage(" ")
            if(arcpy.GetParameter(6)):
                    arcpy.AddMessage("User elected to create display layer")
                    try:
                            x_coords = "Longitude"
                            y_coords = "Latitude"
                            z_coords = "POINT_Z"
                            out_Layer = "Data_Points"
                            saved_Layer = r"Data_Points"

                            # Set the spatial reference
                            spRef = r"Coordinate Systems\Geographic Coordinate System\World\WGS 1984"

                            # Make the XY event layer...
                            for i in range(1,10):
                                    try:
                                            arcpy.MakeXYEventLayer_management(tablePathNew, x_coords, y_coords, out_Layer, spRef)
                                            arcpy.SaveToLayerFile_management(out_Layer, saved_Layer)
                                            mxd = arcpy.mapping.MapDocument("CURRENT")
                                            dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                            addlayer = arcpy.mapping.Layer(out_Layer)
                                            break
                                    except:
                                            if("ERROR 000725" in str(arcpy.GetMessage(3)) ):
                                                    saved_Layer_new = saved_Layer+"_"+str(i)
                                                    out_Layer_new = out_Layer+"_"+str(i)						
                                                    try:
                                                        arcpy.MakeXYEventLayer_management(tablePathNew, x_coords, y_coords, out_Layer_new, spRef)
                                                        arcpy.SaveToLayerFile_management(out_Layer_new, saved_Layer_new)
                                                        mxd = arcpy.mapping.MapDocument("CURRENT")
                                                        dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                                        addlayer = arcpy.mapping.Layer(out_Layer_new)
                                                        break
                                                    except:
                                                        if(i==1):
                                                            arcpy.AddMessage("Data Layer "+ out_Layer + " already exists, trying: "+out_Layer_new)
                                                            arcpy.AddMessage(" ")
                                                        else:
                                                            arcpy.AddMessage("Data Layer "+ out_Layer_new + " already exists, trying: "+out_Layer+"_"+str(i+1))
                                                            arcpy.AddMessage(" ")

                            # Save to a layer file			
                            arcpy.mapping.AddLayer(dataFrame, addlayer, "TOP")

                    except:
                            #if error occured print message to screen
                            arcpy.AddMessage(arcpy.GetMessages())
            else:
                    arcpy.AddMessage("User elected not to create display layer")
        else:
                arcpy.AddWarning("Please ensure your api key is entered correctly")
#endregion

#region data View

if (tablePath.endswith('.csv')) or (tablePath.endswith('.txt')):
    count = 0
    for field in fieldName:
        if field == address:
            addressIndex = count
        if field == city:
            cityIndex = count
        if field == state:
            stateIndex = count
        if field == zip:
            zipIndex = count
        count = count +1
    arcpy.AddWarning("Processing csv file...")
    csvPathOut = tablePath[:-4]
    if tablePath.endswith('.txt'):
        newTable = csvPathOut +'_Results.txt'
    else:
        newTable = csvPathOut +'_Results.csv'
    with open(tablePath,'r') as csvinput:
        with open(newTable,'w') as csvoutput:
            totalRows = int(arcpy.GetCount_management(tablePath).getOutput(0))
            arcpy.AddWarning("Processing "+str(totalRows)+" total records...")
            #this sets up the progress bar
            arcpy.SetProgressor("step", "Copying shapefiles to geodatabase...",0, totalRows, 1)
            start_time = time.time()
            success = 1
            writer = csv.writer(csvoutput, lineterminator='\n')

            all = []
            if tablePath.endswith('.txt'):
                reader = csv.reader(csvinput, delimiter='\t')
            else:
                reader = csv.reader(csvinput)
            row = next(reader)
            for field in fields_to_add:
                if field not in fieldName:
                    row.append(field)
            all.append(row)
            for row in reader:
                arcpy.SetProgressorLabel("Loading {0}...".format(row))
                #myList = list(row)
                #arcpy.AddError(myList)
                url = "https://geoservices-api.tamu.edu/Services/Geocode/WebService/GeocoderWebServiceHttpNonParsed_V04_01.aspx?"

                payload = "streetAddress="+str(row[addressIndex])+"&city="+str(row[cityIndex])+"&state="+str(row[stateIndex])+"&zip="+str(row[zipIndex])+"&apikey="+apikey+"&format=csv&census=true&censusYear=2010&notStore=false&version=4.01&source=Esri"

                headers = {
                        'accept': "application/x-www-form-urlencoded",
                        'content-type': "application/x-www-form-urlencoded",
                        'access-control-allow-origin': "*"
                        }

                response = requests.request("POST", url, data=payload, headers=headers)
                length = len(response.text)
                if length>0:
                    responseList = response.text.split(',')
                else:
                    arcpy.AddWarning("There are 0 results")
                    arcpy.AddError("This is likely due to an invalid api key")
                    success = 0
                    sys.exit(1)
                responseListOut = []
                row.append(responseList[3])
                row.append(responseList[4])
                row.append(responseList[11])
                row.append(responseList[9])
                if(arcpy.GetParameter(7) and responseList[9] == "Success"):
                    row.append(responseList[40])
                    row.append(responseList[41])
                    row.append(responseList[42])
                    row.append(responseList[43])
                    row.append(responseList[44])
                    row.append(responseList[45])
                    row.append(responseList[46])
                    row.append(responseList[47])
                    row.append(responseList[48])
                    row.append(responseList[49])
                    row.append(responseList[50])
                all.append(row)
                arcpy.SetProgressorPosition()

            writer.writerows(all)
            end_time = (time.time()-start_time)
            if(success == 1):
                arcpy.AddMessage(" ")
                arcpy.AddMessage("Processing complete: " + str(totalRows) + " records")
                arcpy.AddMessage("Processing speed: " + str("%.2f" % (totalRows/end_time)) + " records/per second")
                arcpy.AddMessage(" ")
                if(arcpy.GetParameter(6)):
                    arcpy.AddMessage("User elected to create display layer")
                    try:
                        x_coords = "Longitude"
                        y_coords = "Latitude"
                        z_coords = "POINT_Z"
                        out_Layer = "Data_Points"
                        saved_Layer = r"Data_Points"

                        # Set the spatial reference
                        spRef = r"Coordinate Systems\Geographic Coordinate System\World\WGS 1984"

                        # Make the XY event layer...
                        for i in range(1,10):
                            try:
                                arcpy.MakeXYEventLayer_management(newTable, x_coords, y_coords, out_Layer, spRef)
                                arcpy.SaveToLayerFile_management(out_Layer, saved_Layer)
                                mxd = arcpy.mapping.MapDocument("CURRENT")
                                dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                addlayer = arcpy.mapping.Layer(out_Layer)
                                break
                            except:
                                if("ERROR 000725" in str(arcpy.GetMessage(3)) ):
                                    saved_Layer_new = saved_Layer+"_"+str(i)
                                    out_Layer_new = out_Layer+"_"+str(i)
                                    try:
                                        arcpy.MakeXYEventLayer_management(newTable, x_coords, y_coords, out_Layer_new, spRef)
                                        arcpy.SaveToLayerFile_management(out_Layer_new, saved_Layer_new)
                                        mxd = arcpy.mapping.MapDocument("CURRENT")
                                        dataFrame = arcpy.mapping.ListDataFrames(mxd, "*")[0]
                                        addlayer = arcpy.mapping.Layer(out_Layer_new)
                                        break
                                    except:
                                        if(i==1):
                                            arcpy.AddMessage("Data Layer "+ out_Layer + " already exists, trying: "+out_Layer_new)
                                            arcpy.AddMessage(" ")
                                        else:
                                            arcpy.AddMessage("Data Layer "+ out_Layer_new + " already exists, trying: "+out_Layer+"_"+str(i+1))
                                            arcpy.AddMessage(" ")

                        # Save to a layer file
                        arcpy.mapping.AddLayer(dataFrame, addlayer, "TOP")
                    except:
                        #if error occured print message to screen
                        arcpy.AddWarning("It could not added layer")
                        arcpy.AddMessage(arcpy.GetMessages())
                else:
                    arcpy.AddMessage("User elected not to create display layer")
            else:
                arcpy.AddWarning("Please ensure your api key is entered correctly")
                sys.exit(1)
        arcpy.ResetProgressor()
    sys.exit(1)
#endregion

else:
    arcpy.AddError("This is not a valid file")
#This resets the progress bar
arcpy.ResetProgressor()
sys.exit(1)




